// This file is part of the brownmove simulation package
// 
// V1.2  Apr. 2011
// 
// (C) 2003 - 2011 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove



#ifndef ESTATSHAPE_H
#define ESTATSHAPE_H

#include <string>
#include <vector>
#include <iostream>
#include <string>
#include <sstream>

#include "types.h"
#include "constants.h"
#include "vectormath.h"

#include "shape.h"


namespace brown{
    /**
     *@author jni, tg
     */
    
	//! EstatShape handles the interactions between charges on the particles.
    class EstatShape : public Shape {
	// friend class Wall;				// can't recall why this was neccessary at some point
	
	public:
	EstatShape();
	EstatShape(const EstatShape &other);
	~EstatShape();
	
	
	/** restores the parts' (absolute and relative) positions from fullPosition */
	virtual void recalculate(const fullPosition &p);
	
	/** returns some string for testing */
	virtual std::string describe();
	
	//! adds another charge with position and magnitude to the vector 
	//! Also calculates the relatiev position of the charge from the molecules center given
	void addCharge(chargePos cp, const position &moleculeCenter);
	
	
	/** set the pointer to an analytic function describing the interaction between a wall and a molecule */
	void setInteraction(functionPointer &fp) {
	    interAction = fp;
	}
	
	//! return the function pointer to an analytical interaction
	functionPointer getInteraction() {
	    return(interAction);
	}
	
	
	//! calculate the forces and torques from the charges of this and the other EstatShape
	void calculateImpact(fullForce &myForce, fullForce &otherForce, EstatShape *es, const bool isPeriodic[3], const doublevec &cellSize);
	
	//! returns the number of charges
	int size() { return(charges.size()); }
	
	//! Dumps the known infos about charges in browndef format
	void dumpStructure();

	
	private:
	//! The relative positions of the charges and their charge
	vector<chargePos> charges;
	//! Keep track of the absolute positions of the charges, updated when shape is moved
	vector<chargePos> actual_charges;
	//! pointer to an analytic function for a wall-point charge interaction
	functionPointer interAction;

    };
}

#endif
