// This file is part of the brownmove simulation package
// 
// V1.2  Apr. 2011
// 
// (C) 2003 - 2011 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove



#ifndef MARKERSHAPE_H
#define MARKERSHAPE_H

#include <string>
#include <vector>
#include <iostream>
#include <string>
#include <sstream>

#include "types.h"
#include "constants.h"
#include "vectormath.h"

#include "shape.h"


namespace brown{
    /**
     *@author tg
     */
    
	//! MarkerShape is only relevant for analysis: arbitrarily placed Markers allow to extract distances etc.
    class MarkerShape : public Shape {
	
	public:
	MarkerShape() { };
	MarkerShape(const MarkerShape &other);
	~MarkerShape();
	
	
	/** restores the parts' (absolute and relative) positions from fullPosition */
	virtual void recalculate(const fullPosition &p);
	
	/** returns some string for testing */
	virtual std::string describe();
	
	//! adds another marker with position and index to the vector 
	//! Also calculates the relatiev position of the marker from the molecules center given
	void addMarker(markerPos mp, const position &moleculeCenter);
		
	//! returns the number of markers
	int size() { return(markers.size()); }
	
	//! Dumps the known infos about markers in browndef format
	void dumpStructure();

	//! Count and dump the actual positions and indices of the markers to stringstream raus
	int dumpMarkers(char *prefix, std::string *raus);

	
	private:
	//! The relative positions of the markers and their indices
	vector<markerPos> markers;
	//! Keep track of the absolute positions of the markers, updated when shape is moved
	vector<markerPos> actual_markers;

    };
}

#endif
