// This file is part of the brownmove simulation package
// 
// V1.2  Apr. 2011
// 
// (C) 2003 - 2011 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove



#include "globalstuff.h"

// using namespace brown;

namespace brown {
	// some global variables for global stuff
	// Q: Should we add the timestep here? => No, as long as it does not change during the simulation
	// 
	double KAPPA = 0.99;			// for physiological salt concentration of 0.09M
	double EPS_H2O = 78.0;
	double BETA = 1.0/(K_B*293.0);	// default temp = 20 degrees
	
	double HBOND_DEPTH = 25.0;		// average H-bond: 25 kJ/mol
	double HBOND_LENGTH = 0.3;		// length of an average H bond (between donor-hydrogen and acceptor-atom)
	double HBOND_WIDTH = 0.05;		// potential width in nm
	double HBOND_CUTOFF = 1.5;		// ignore H bond if donor and acceptor are further apart


	

	// Read next line, chop off leading whitespace and ignore comment lines
	int myGetLine(FILE *fp_in, char *newLine, const int maxLen, std::map<std::string, std::string> *constants, bool noLowerCase)
	{
		int c;
		int count = 0;
		bool leadingWhite = true;
		bool ignoreLine = true;
		char myLine [2049];
		
		while(ignoreLine) {
			leadingWhite = true;
			count = 0;
			
			c = fgetc(fp_in);
			
			while(c != '\n') {
				if(c == EOF) {
					return(c); }
				
				if((c != ' ' && c != '\t')  || leadingWhite == false) {
					leadingWhite = false;
					myLine[count] = c;
					count++;
					if(count >= maxLen) {
						myLine[count-1] = '\0';
						return(0);
					}
				}
				c = fgetc(fp_in);
			}
			myLine[count] = '\0';
			if(count > 0 && myLine[0] != '#') {
				ignoreLine = false; }
		}
		
		// std::cerr << "myGetLine: myLine = >>" << myLine << "<<" << std::endl;
		
		// check for replacements
		int i, k, it, itt;
		bool inConst = false;
		char target[2048];
		
		k = 0;
		i = 0;
		while(myLine[i] != '\0' || inConst)
		{
			char next = myLine[i];
			
			if(inConst)
			{
				// check whether a replacement is known
				if(constants->count(std::string(target))) {
					for(itt=0; itt<(*constants)[target].size(); itt++) {
						newLine[k] = (*constants)[target].c_str()[itt];
						k++;
					}
					inConst = false;
					i--;				// restart from current char
				}
				else  // keep reading target string
				{
					// std::cerr << "myGetLine: no replacement found for constant >>" << target << "<<" << std::endl;
					
					if(next != ' ' && next != '\t') {
						target[it] = next;
						target[it+1] = '\0';
						it++;
					} 
					else {	// replacement not known - exit!
						std::cerr << "myGetLine: constant >>" << target << "<< is undefined in line >>" << myLine << "<<" << std::endl;
						exit(-4);
					}
					// std::cerr << "myGetLine: target = >>" << target << "<<" << std::endl;
				}
			}
			
			else if(next == '$')  {	// start reading target
				inConst = true;
				it = 0;
				target[it] = '\0';
			} 
			else {				// just copy
				newLine[k] = next;
				k++;
			}
			
			i++;
		}
		newLine[k] = '\0';
		
		// std::cerr << "myGetLine: nearly finished, newLine = >>" << newLine << "<<" << std::endl;
		
		// convert 1st word to lower case
		if(!noLowerCase) {
			i=0;
			while(newLine[i] != ' ' && newLine[i] != '\t') {
				newLine[i] = tolower(newLine[i]);
				i++;
			}
		}
		
		// std::cerr << "myGetLine: finished,       newLine = >>" << newLine << "<<" << std::endl;

		return(0);
	}
}


