// This file is part of the brownmove simulation package
// 
// V1.2  Apr. 2011
// 
// (C) 2003 - 2011 Tihamer Geyer
// tihamer.geyer@bioinformatik.uni-saarland.de
// 
// Check for Updates at the Project Homepage 
// http://service.bioinformatik.uni-saarland.de/brownmove



#include "molecule.h"
#include "wall.h"

namespace brown{

	
	Molecule::Molecule(std::string name, long int num) :InteractingBody(name, num)
	{
		insertionTime.when = 0;
		insertionTime.where = "";
		takeoutTime.when = 0;
		takeoutTime.where = "";
	}

	
	Molecule::Molecule(const Molecule &other, long int num)
 	:InteractingBody(other)
	{
		// set ID
		id.mclass = other.id.mclass;
		id.mnumber = num;				// set new id number
		
		// Copy come and gone
		insertionTime.when = other.insertionTime.when;
		insertionTime.where = other.insertionTime.where;
		takeoutTime.when = other.takeoutTime.when;
		takeoutTime.where = other.takeoutTime.where;
	}
	
	
	Molecule::~Molecule(){
		// recursively tear down all ingredients
		Gestalten.clear();
		bondVector.clear();
	}
	
	
	void Molecule::teleport(const fullPosition &p){
		InteractingBody::teleport (p);
	}

	
	void Molecule::positionToSnapshot(FILE *fp) 
	{
		char eiDi[200];
		int i;
		
		for(i=0; i<id.mclass.size(); i++) {
			eiDi[i] = id.mclass[i];
		}
		eiDi[i] = '\0';
		
		for(std::vector<Gestalt>::iterator it = Gestalten.begin(); it != Gestalten.end(); it++) {
			(*it).positionToSnapshot(fp, eiDi, id.mnumber);
		}
	}
	


	void Molecule::immobilize(long int n) {
		for(std::vector<Gestalt>::iterator it = Gestalten.begin(); it != Gestalten.end(); it++) {
			(*it).immobilize(n);
		}
	}
	
	long int Molecule::getImmobilized() {
		return(Gestalten.begin()->getImmobilized());
	}


	void Molecule::whatIsPushing() {
		for(std::vector<Gestalt>::iterator it1 = Gestalten.begin(); it1 != Gestalten.end(); ++it1) {
			(*it1).calculateExternalForce();
		}
	}

	
	void Molecule::setHICutOff(double c, double s) {
		for(std::vector<Gestalt>::iterator it = Gestalten.begin(); it != Gestalten.end(); it++) {
			if((*it).getHiShape()) {
				((*it).getHiShape())->setHICutOff(c, s);
			}
		}
	}
}
